/*****************************************************************************
**
** plugin0.h
** $Header: /home/omer/work/cvs_repository/Hebrew/include/plugin0.h,v 1.5 1999/02/06 16:49:27 omer Exp $
**
** Level 0 I18N text rendering plugin interface specifications, relevant
** to plugin writers.
**
** Copyright (C) 1999  Omer Zak
**
**
** This library is free software; you can redistribute it and/or
** modify it under the terms of the GNU Library General Public
** License as published by the Free Software Foundation; either
** version 2 of the License, or (at your option) any later version.
**
** This library is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
** Library General Public License for more details.
**
** You should have received a copy of the GNU Library General Public
** License along with this library; if not, write to the 
** Free Software Foundation, Inc., 59 Temple Place - Suite 330, 
** Boston, MA  02111-1307  USA.
**
******************************************************************************
**
** Bug reports and fixes - to omerz@actcom.co.il
** Note:  by sending unsolicited commercial/political/religious/MailPush
**        E-mail messages (known also as "spam") to any E-mail address
**        mentioned in this file, you irrevocably agree to pay the
**        receipient US$500.- (plus any legal expenses incurred while
**        trying to collect the amount due) per unsolicited
**        commercial/political/religious/MailPush E-mail message - for
**        the service of receiving your E-mail message.
**
*****************************************************************************/

#ifndef _PLUGIN0_H_
#define _PLUGIN0_H_

#include <stdlib.h>
#include "structs0.h"
#include "plugimp0.h"

#ifdef __cplusplus
extern "C" {
#endif


/*****************************************************************************
** Dynamic library initialization and finalization functions                **
*****************************************************************************/
void _init(void);  /* To be supplied by the plugin writer */
void _fini(void);  /* To be supplied by the plugin writer */


/*****************************************************************************
** The following is the main connection point between the plugin
** and the client software.
*****************************************************************************/

RETURN_FUNCTION_POINTERS return_function_pointers;
  /* This function is to be implemented by the plugin writer. */


/*****************************************************************************
** Plugin configuration                                                     **
** The plugin writer must implement three procedures:                       **
** parse_config() - to parse a string and convert it into internal          **
**     configuration data structure.                                        **
** validate_config() - to validate the internal configuration data          **
**     structure.                                                           **
** delete_config() - to free any memory structures in the internal          **
**     configuration data structure.
**                                                                          **
** Note:                                                                    **
** Sane default behavior should be defined - if the input                   **
** configuration string is NULL or empty, then the output configuration     **
** structure pointer is to be set to NULL.                                  **
** Furthermore, the rest of the plugin code should invoke default           **
** behavior when the pointer to the configuration structure is NULL.        **
*****************************************************************************/

  /* To be supplied by the plugin writer */
typedef E_RENDER_RETURN_CODES
(PARSE_CONFIG)(const char *in_config_str,
	                           /* Configuration string - typically given
				   ** as a command line argument or retrieved
				   ** from a resource file. */
	       MALLOC *in_malloc_func,
	                           /* Function for allocating
				   ** memory, if any is needed for the
				   ** *out_plugin_config_p_p data
				   ** structure which holds configuration
				   ** information.
				   ** malloc() will be used if the
				   ** argument is NULL. */
	       FREE *in_free_func, /* Function for freeing
				   ** memory allocated by
				   ** in_malloc_func().
				   ** free() will be used if
				   ** the argument is NULL. */
	       void **out_plugin_config_p_p);
                                   /* Pointer to opaque configuration data
				   ** structure, which will be used by
				   ** other procedures.  If necessary,
				   ** the data structure will contain also
				   ** a pointer to a procedure to free
				   ** allocated memory (initialized
				   ** from in_free_func). */
PARSE_CONFIG parse_config;

  /* To be supplied by the plugin writer */
BOOL
validate_config(const void *in_plugin_config_p);
                                   /* Pointer to plugin configuration opaque
				   ** data structure, whose contents are to
				   ** be validated. */

  /* To be supplied by the plugin writer */
FREE delete_config;


/*****************************************************************************
** Procedures which manipulate SRenderedText and its subordinate data       **
** structures.                                                              **
*****************************************************************************/

/*****************************************************************************
** Fragment manipulation code
*****************************************************************************/

/*****************************************************************************
** Fragment constructor
** NULL is returned if memory allocation attempt failed.
**
** Not all SRenderedFragment fields and data structures are initialized
** by this constructor.
** The calling software is responsible for initializing some of them.
*****************************************************************************/

SRenderedFragment *
construct_fragment(const SRenderingContext *in_rc_p,
		   size_t in_frag_length,
		   long in_orig_frag_attributes,
		   long in_added_frag_attributes,
		   size_t in_first_orig_char_index,
		   size_t in_orig_frag_length);


/*****************************************************************************
** Line manipulation code                                                   **
*****************************************************************************/

/*****************************************************************************
** Line constructor
** NULL is returned if memory allocation attempt failed.
*****************************************************************************/
SRenderedLine *
construct_line(const SRenderingContext *in_rc_p,
	       size_t in_screen_row_width);

/*****************************************************************************
** Append a new fragment to the end of a line
*****************************************************************************/

E_RENDER_RETURN_CODES
line_append_fragment(const SRenderingContext *in_rc_p,
		     SRenderedLine *in_out_line_p,
		     SRenderedFragment *in_new_frag_p);

/*****************************************************************************
** Delete all fragments from a line, starting at a certain fragment
*****************************************************************************/

E_RENDER_RETURN_CODES
line_delete_fragments(const SRenderingContext *in_rc_p,
		      SRenderedLine *in_out_line_p,
		      size_t in_first_frag_to_free);
				  /* Index of first fragment to be freed
				  ** (the index of 1st fragment is 0). */



/*****************************************************************************
** SRenderedText manipulation code                                          **
*****************************************************************************/

/*****************************************************************************
** Append a new line to the end of SRenderedText
*****************************************************************************/

E_RENDER_RETURN_CODES
text_append_line(SRenderedText *in_out_rt_p,
				  /* Already contains pointer to the
				  ** current rendering context. */
		 SRenderedLine *in_new_line_p);

/*****************************************************************************
** Delete all lines from a SRenderedText, starting at a certain line.
*****************************************************************************/

E_RENDER_RETURN_CODES
text_delete_lines(SRenderedText *in_out_rt_p,
				  /* Already contains pointer to the
				  ** current rendering context. */
		  size_t in_first_line_to_free);
				  /* Index of first line to be freed
				  ** (the index of 1st line is 0). */

/*****************************************************************************
** Determine the offset in original text, which corresponds to 1st glyph
** in a line - for a line, which does not exist yet.
** Information in the previous line is used for this purpose.
*****************************************************************************/

E_RENDER_RETURN_CODES
locate_origtext_offset(const SRenderedText *in_rt_p,
				  /* Already contains pointer to the
				  ** current rendering context. */
		       size_t in_line,
				  /* Index of line for which we want the
				  ** offset in the original text. */
		       size_t *out_offset_p);
                                  /* Return value - contains the desired
				  ** offset.  If the previous line does
				  ** not exist, an error return code is
				  ** returned. */

/*****************************************************************************
** Notify the plugin about a modification in the original text.
** The client-accessible function encapsulates a call to the plugin-provided
** function called p_notify_modification() and with the same arguments.
**
** The basic idea is that a change consists of deletion of a text segment
** starting at offset N and having length L1, followed by insertion
** of another text segment starting at the same offset and having length L2.
** Either text segment may have zero length.
** Thus, the same mechanism is used to describe both text replacements,
** insertions and deletions.
*****************************************************************************/

typedef E_RENDER_RETURN_CODES
(P_NOTIFY_MODIFICATION)(SRenderedText *in_out_text_p,
                                  /* The rendered text data structure, which
				  ** is affected by the announced change. */
		      size_t in_change_is_at,
		                  /* The offset in the original text
                                  ** in which the change took place. */
		      size_t in_deleted_chars,
		                  /* Number of deleted characters */
		      size_t in_inserted_chars,
		                  /* Number of inserted characters */

		      GET_ORIG_TEXT_FRAGMENT *in_get_frag_func,
			          /* Client-provided callback for
			          ** retrieving original text
			          ** fragments. */
		      void *in_client_data_p,
			          /* Client data, which is passed as
			          ** it is to the callback. */

		      size_t *out_first_modified_line_p);
		                  /* Reply to the client software - which is
				  ** the first rendered line to be affected
				  ** by the change. */

P_NOTIFY_MODIFICATION p_notify_modification;

/*****************************************************************************
** Retrieve a line of rendered glyphs.
** Most plugin writers will probably implement the entire rendering
** algorithm in this procedure, because this is the right place for
** lazy execution of the rendering algorithm.
*****************************************************************************/

typedef E_RENDER_RETURN_CODES
(P_RETRIEVE_RENDERED_LINE)(SRenderedText *in_out_text_p,
                                  /* The rendered text data structure, which
				  ** is affected by the announced change. */
			 size_t in_desired_visual_row,
			          /* Index of the line which the client
				  ** software wants to retrieve. */
			 GET_ORIG_TEXT_FRAGMENT *in_get_frag_func,
			          /* Client-provided callback for
			          ** retrieving original text
			          ** fragments. */
			 void *in_frag_client_data_p,
			          /* Client data, which is passed as
			          ** it is to the in_get_frag_func
				  ** callback. */
			 GET_RENDERED_LINE_SIZE *in_get_line_size_func,
			          /* Client-provided callback for
				  ** retrieving desired rendered line
				  ** size. */
			 void *in_size_client_data_p,
			          /* Client data, which is passed as
			          ** it is to the in_get_line_size_func
				  ** callback. */
			 /* Output arguments */
			 size_t *out_line_length_p,
			         /* The array length is in glyphs
				 ** (which could be multi-byte). */
			 void **out_glyphs_arr_p);

P_RETRIEVE_RENDERED_LINE p_retrieve_rendered_line;

/****************************************************************************/

#ifdef __cplusplus
}; /* end of extern "C" */
#endif

#endif /* _PLUGIN0_H_ */

/* End of plugin0.h */
